from mbientlab.metawear import MetaWear, libmetawear, parse_value
from mbientlab.metawear.cbindings import *
from time import sleep
from threading import Event

import copy
import signal
import sys

device = MetaWear(sys.argv[1])
device.deserialize()
device.connect()
print("Connected")

event = Event()
init_handler = FnVoid_VoidP_Int(lambda caller, status: event.set())
libmetawear.mbl_mw_metawearboard_initialize(device.board, init_handler)
event.wait()

if libmetawear.mbl_mw_metawearboard_is_initialized(device.board) != 0:
    device.serialize()

    print("model = " + str(libmetawear.mbl_mw_metawearboard_get_model_name(device.board)))
    libmetawear.mbl_mw_settings_set_connection_parameters(device.board, 7.5, 7.5, 0, 6000);
    libmetawear.mbl_mw_acc_set_odr(device.board, 50.0);
    libmetawear.mbl_mw_acc_set_range(device.board, 4.0);
    libmetawear.mbl_mw_acc_write_acceleration_config(device.board);

    samples = []
    def counter(data):
        samples.append(copy.deepcopy(parse_value(data)))

    data_handler = FnVoid_DataP(counter)
    signal = libmetawear.mbl_mw_acc_get_packed_acceleration_data_signal(device.board);
    libmetawear.mbl_mw_datasignal_subscribe(signal, data_handler)

    libmetawear.mbl_mw_acc_enable_acceleration_sampling(device.board);
    libmetawear.mbl_mw_acc_start(device.board);
    print("Sampling data")

    sleep(30.0)

    libmetawear.mbl_mw_acc_stop(device.board);
    libmetawear.mbl_mw_acc_disable_acceleration_sampling(device.board);

    for s in samples:
        print(str(s))
    print("collected samples = %d" % (len(samples)))

    

device.disconnect()
print("disconnected")



from mbientlab.warble import *

from collections import deque
from ctypes import *
from time import sleep
from threading import Event

import sys

e = Event()

print(sys.argv[1])
gatt = Gatt(sys.argv[1])
gatt.connect_async(lambda error: e.set())
e.wait()

print("connected")
e.clear()

samples = 0
def value_changed(value):
    print(str(value))
    global samples
    samples += 1
notify = gatt.find_characteristic("326a9006-85cb-9195-d9dd-464cfbbae75a")
notify.on_notification_received(value_changed)
notify.enable_notifications_async(lambda error: e.set())
e.wait()

cmd = gatt.find_characteristic("326a9001-85cb-9195-d9dd-464cfbbae75a")
values = deque([
    [0x1, 0x80],
    [0x2, 0x80],
    [0x3, 0x80],
    [0x4, 0x80],
    [0x5, 0x80],
    [0x6, 0x80],
    [0x7, 0x80],
    [0x8, 0x80],
    [0x9, 0x80],
    [0xa, 0x80],
    [0xb, 0x80],
    [0xc, 0x80],
    [0xd, 0x80],
    [0xf, 0x80],
    [0x10, 0x80],
    [0x11, 0x80],
    [0x12, 0x80],
    [0x13, 0x80],
    [0x14, 0x80],
    [0x15, 0x80],
    [0x16, 0x80],
    [0x17, 0x80],
    [0x18, 0x80],
    [0xfe, 0x80]
])
def write_values():
    if len(values) == 0:
        e.set()
    else:
        def completed(error):
            if error != None:
                print(error)
            write_values()
        cmd.write_async(values.popleft(), completed)

chars = deque([
    "00002a26-0000-1000-8000-00805f9b34fb",
    "00002a24-0000-1000-8000-00805f9b34fb",
    "00002a27-0000-1000-8000-00805f9b34fb",
    "00002a29-0000-1000-8000-00805f9b34fb",
    "00002a25-0000-1000-8000-00805f9b34fb"
])
def read_gatt_chars():
    if len(chars) == 0:
        write_values()
    else:
        gattchar = gatt.find_characteristic(chars.popleft())
        if gattchar != None:
            def completed(value, error):
                if error == None:
                    print("%s: %s" % (gattchar.uuid, bytearray(value).decode('ascii')))
                    read_gatt_chars()
                else:
                    print("%s: Error reading gatt char (%s)" % (gattchar.uuid, error))

            gattchar.read_value_async(completed)
        else:
            print("%s: does not exist" % gattchar.uuid)
            read_gatt_chars()

e.clear()
read_gatt_chars()
e.wait()

e.clear()
#cmd.write_without_resp_async([0x11, 0x09, 0x06, 0x00, 0x06, 0x00, 0x00, 0x00, 0x58, 0x02], lambda error: e.set())
#e.wait()

#sleep(2.0)
#e.clear()

#values = deque([
#    [0x03, 0x03, 0x29, 0x0c], 
#    [0x03, 0x02, 0x01, 0x00],
#    [0x03, 0x04, 0x01],
#    [0x13, 0x03, 0x29, 0x00],
#    [0x13, 0x05, 0x01],
#    [0x13, 0x02, 0x01, 0x00],
#    [0x03, 0x01, 0x01],
#    [0x13, 0x01, 0x01]
#])

sleep(30.0)

print("samples: " + str(samples))

gatt.on_disconnect(lambda status: e.set())
cmd.write_async([0xfe, 0x01], lambda status: None)

e.wait()
print("disconnected")



from common import TestMetaWearBase, to_string_buffer
from mbientlab.metawear.cbindings import *
from ctypes import create_string_buffer

class AccelerometerBoschBase:
    class TestOrientation(TestMetaWearBase):
        def test_enable(self):
            self.libmetawear.mbl_mw_acc_bosch_enable_orientation_detection(self.board)
            self.assertEqual(self.command, [0x03, 0x0f, 0x01, 0x00])

        def test_disable(self):
            self.libmetawear.mbl_mw_acc_bosch_disable_orientation_detection(self.board)
            self.assertEqual(self.command, [0x03, 0x0f, 0x00, 0x01])

        def test_handle_response(self):
            tests= [
                {
                    'expected': SensorOrientation.FACE_UP_LANDSCAPE_RIGHT,
                    'response': [0x03, 0x11, 0x07],
                    'name': 'fulr'
                },
                {
                    'expected': SensorOrientation.FACE_UP_PORTRAIT_UPRIGHT,
                    'response': [0x03, 0x11, 0x01],
                    'name': 'fupu'
                },
                {
                    'expected': SensorOrientation.FACE_UP_PORTRAIT_UPSIDE_DOWN,
                    'response': [0x03, 0x11, 0x03],
                    'name': 'fupd'
                },
                {
                    'expected': SensorOrientation.FACE_UP_LANDSCAPE_LEFT,
                    'response': [0x03, 0x11, 0x05],
                    'name': 'full'
                },
                {
                    'expected': SensorOrientation.FACE_DOWN_LANDSCAPE_RIGHT,
                    'response': [0x03, 0x11, 0x0f],
                    'name': 'fdlr'
                },
                {
                    'expected': SensorOrientation.FACE_DOWN_LANDSCAPE_LEFT,
                    'response': [0x03, 0x11, 0x0d],
                    'name': 'fdll'
                },
                {
                    'expected': SensorOrientation.FACE_DOWN_PORTRAIT_UPRIGHT,
                    'response': [0x03, 0x11, 0x09],
                    'name': 'fdpu'
                },
                {
                    'expected': SensorOrientation.FACE_DOWN_PORTRAIT_UPSIDE_DOWN,
                    'response': [0x03, 0x11, 0x0b],
                    'name': 'fdpd'
                }
            ]

            for test in tests:
                with self.subTest(odr= test['name']):
                    orientation = self.libmetawear.mbl_mw_acc_bosch_get_orientation_detection_data_signal(self.board)
                    self.libmetawear.mbl_mw_datasignal_subscribe(orientation, None, self.sensor_data_handler)
                    
                    self.notify_mw_char(to_string_buffer(test['response']))
                    self.assertEqual(self.data_int32.value, test['expected'])

class TestAccelerometerBmi160Orientation(AccelerometerBoschBase.TestOrientation):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_RPRO_BOARD

        super().setUp()

class TestAccelerometerBma255rientation(AccelerometerBoschBase.TestOrientation):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_ENV_BOARD

        super().setUp()


from .cbindings import *
from ctypes import CDLL

import os
import platform

if (platform.system() == 'Windows'):
    _so_path = os.path.join(os.path.dirname(__file__), 'MetaWear.Win32.dll')
elif (platform.system() == 'Linux'):
    _so_path = os.path.join(os.path.dirname(__file__), 'libmetawear.so')
else:
    raise RuntimeError("MetaWear Python SDK is not supported for '%s'" % platform.system())

libmetawear= CDLL(_so_path)
init_libmetawear(libmetawear)

from .metawear import MetaWear

def parse_value(p_data):
    """
    Helper function to extract the value from a Data object.  If you are storing the values to be used at a later time, 
    call copy.deepcopy preserve the value.  You do not need to do this if the underlying type is a native type or a byte array
    @params:
        p_data      - Required  : Pointer to a Data object
    """
    if (p_data.contents.type_id == DataTypeId.UINT32):
        return cast(p_data.contents.value, POINTER(c_uint)).contents.value
    elif (p_data.contents.type_id == DataTypeId.INT32 or p_data.contents.type_id == DataTypeId.SENSOR_ORIENTATION):
        return cast(p_data.contents.value, POINTER(c_int)).contents.value
    elif (p_data.contents.type_id == DataTypeId.FLOAT):
        return cast(p_data.contents.value, POINTER(c_float)).contents.value
    elif (p_data.contents.type_id == DataTypeId.CARTESIAN_FLOAT):
        return cast(p_data.contents.value, POINTER(CartesianFloat)).contents
    elif (p_data.contents.type_id == DataTypeId.BATTERY_STATE):
        return cast(p_data.contents.value, POINTER(BatteryState)).contents
    elif (p_data.contents.type_id == DataTypeId.BYTE_ARRAY):
        p_data_ptr= cast(p_data.contents.value, POINTER(c_ubyte * p_data.contents.length))

        byte_array= []
        for i in range(0, p_data.contents.length):
            byte_array.append(p_data_ptr.contents[i])
        return byte_array
    elif (p_data.contents.type_id == DataTypeId.TCS34725_ADC):
        return cast(p_data.contents.value, POINTER(Tcs34725ColorAdc)).contents
    elif (p_data.contents.type_id == DataTypeId.EULER_ANGLE):
        return cast(p_data.contents.value, POINTER(EulerAngles)).contents
    elif (p_data.contents.type_id == DataTypeId.QUATERNION):
        return cast(p_data.contents.value, POINTER(Quaternion)).contents
    elif (p_data.contents.type_id == DataTypeId.CORRECTED_CARTESIAN_FLOAT):
        return cast(p_data.contents.value, POINTER(CorrectedCartesianFloat)).contents
    elif (p_data.contents.type_id == DataTypeId.OVERFLOW_STATE):
        return cast(p_data.contents.value, POINTER(OverflowState)).contents
    else:
        raise RuntimeError('Unrecognized data type id: ' + str(p_data.contents.type_id))
